==> agora_flutter_webrtc-0.0.2.tar.gz: lib/agora_client.dart <==
import 'dart:async';

import 'package:agora_flutter_webrtc/src/core/exception.dart';

import './src/utils/index.dart';
import './src/utils/logger.dart';
import './src/utils/report.dart';

import './src/core/gateway_client.dart';
import './agora_stream.dart';
import './src/utils/events.dart';

class AgoraClient extends EventEmitter {
  /// The channel profile.
  /// 
  /// Agora Web SDK needs to know the application scenario to apply different optimization methods.
  /// 
  /// Currently SDK supports the following channel profiles:
  /// - "live": Sets the channel profile as live broadcast.
  /// - "rtc": Sets the channel profile as communication. This is used in one-on-one calls or group calls, where all users in the channel can talk freely.
  String mode;

  /// The codec the Web browser uses for encoding.
  /// 
  /// - "vp8": Sets the browser to use VP8 for encoding.
  /// - "h264": Sets the browser to use H.264 for encoding.
  String codec;

  /// App IDs are issued to app developers by Agora to identify the projects and organizations. 
  /// 
  /// After signing up at [Agora Dashboard](https://dashboard.agora.io), you can create multiple projects, and each project will have a unique App ID. See [Getting an App ID](https://docs.agora.io/en/Agora%20Platform/token#getting-an-app-id).
  String appId;

  /// A string that provides a unique channel name for the Agora session. 
  /// 
  /// The length must be within 64 bytes. Supported character scopes:
  /// - 26 lowercase English letters a-z
  /// - 26 uppercase English letters A-Z
  /// - 10 numbers 0-9
  /// - Space
  /// - "!", "#", "$", "%", "&", "(", ")", "+", "-", ":", ";", "<", "=", ".", ">", "?", "@", "[", "]", "^", "_", "{", "}", "|", "~", ","
  String channel;

  /// 当ws断开的时候（State变为DISCONNECTED），如果这个值为true，会自动进行重连。
  bool reconnect = true;
  
  String _token;
  String _sid;
  String _clientId;
  int _uid;
  int _cid;
  int _reconnectCount = 0;

  GatewayClient _gatewayClient;
  Stream _gwEvent;
  Timer _reconTimer;

  /// 本地的流，一个client只能有一个流
  AgoraStream localStream;

  /// 频道内的所有的远端流
  Map<int, AgoraStream> remoteStreams = {};

  /// The user ID, an integer or a string, ASCII characters only. Ensure this ID is unique
  get uid => _uid;

  /// Creates a Client Object
  /// 
  /// This method creates and returns a client object. You can only call this method once each call session.
  AgoraClient({this.appId, this.mode, this.codec}) {
    _clientId = Utils.getUUID().replaceAll(new RegExp(r'-'), '');
    _sid = Utils.getUUID().replaceAll(new RegExp(r'-'), ''); // generate sid
    EventReportManager.register(sid: _sid);
  }

  /// This method joins an AgoraRTC channel.
  /// 
  /// When joining the channel, the [AgoraClient].on("connection-state-change") callbacks are triggered on the local client.
  Future<int> join(String token, String channel, int uid) async {
    Log.debug("[$_clientId]: (client) join with: $token, $channel, $uid");
    _gatewayClient = GatewayClient(
        appId: appId, mode: mode, codec: codec, clientId: _clientId, sid: _sid);
    _gatewayClient.onConnectionStateChange = _onConnectionStateChange;
    this._token = token;
    this.channel = channel;
    var joinRes = await _gatewayClient.join(null, channel, uid);
    _listenGatewayEvent();

    this._uid = joinRes["uid"];
    this._cid = joinRes["cid"];
    
    EventReportManager.report(_sid, "join", {
      "cid": _cid,
      "uid": uid,
      "cname": channel,
      "processId": Log.processId
    });

    return uid;
  }

  /// Publishes a Local Stream
  /// 
  /// This method publishes a local [stream] to the SD-RTN.
  /// When the [stream] is published, the [AgoraClient].on("stream-added") callback is triggered on the remote client.
  Future<AgoraStream> publish(AgoraStream stream) async {
    Log.debug("[$_clientId]: (client) publish");
    stream.streamId = uid;
    localStream = stream;

    EventReportManager.report(_sid, "publish", {"streamId": uid});

    return _gatewayClient.publish(stream);
  }

  /// Unpublishes the Local Stream
  /// 
  /// This method unpublishes the local [stream].
  /// When the [stream] is unpublished, the [AgoraClient].on("stream-removed") callback is triggered on the remote client.
  Future<AgoraStream> unpublish(AgoraStream stream) async {
    Log.debug("[$_clientId]: (client) unpublish");
    _gatewayClient.unpublish(stream);
    stream.streamId = null;
    localStream = null;
    return stream;
  }

  /// Subscribes to a Remote Stream
  /// 
  /// This method enables a user to subscribe to a remote [stream].
  Future<AgoraStream> subscribe(AgoraStream stream) async {
    Log.debug("[$_clientId]: (client) subscribe");
    return _gatewayClient.subscribe(stream);
  }

  /// Unsubscribes from the Remote Stream
  /// 
  /// This method enables the user to unsubscribe from the remote [stream].
  Future<AgoraStream> unsubscribe(AgoraStream stream) async {
    Log.debug("[$_clientId]: (client) unsubscribe");
    return _gatewayClient.unsubscribe(stream);
  }

  /// Leaves an AgoraRTC Channel
  /// 
  /// This method enables a user to leave a channel.
  /// When leaving the channel, the [AgoraClient].on("connection-state-change") callback is triggered on the local client.
  void leave() {
    Log.debug("[$_clientId]: (client) leave");
    _reconTimer?.cancel();
    _gatewayClient?.leave();
  }

  Future _rejoin() async {
    Log.debug("[$_clientId]: (client) rejoin");
    Completer c = Completer<dynamic>();
    var deleyReconTime = _reconnectCount * 2 < 10 ? _reconnectCount * 2 : 30;
    _reconnectCount++;
    _reconTimer = new Timer(Duration(seconds: deleyReconTime), () {
      join(_token, channel, uid).then((res) {
        Log.debug("[$_clientId]: (client) rejoin success");
        c.complete(res);
      }).catchError((err) {
        Log.debug("[$_clientId]: (client) rejoin error");
        _rejoin().then((res) {
          c.complete(res);
        });
      });
    });
    return c.future;
  }

  Future _republish() async {
    Log.debug("[$_clientId]: (client) rejoin");
    return publish(localStream);
  }

  void _onAddAudioStream(Map<String, dynamic> message) {
    Log.debug("[$_clientId]: (client) receive onAddAudioStream: $message");
    var stream = AgoraStream({
      "video": message["video"],
      "audio": message["audio"],
      "streamId": message["uid"],
      "isLocal": false,
    });
    stream.gw = _gatewayClient;
    remoteStreams[stream.getId()] = stream;
    this.emit("stream-added", {"stream": stream});
  }

  void _onAddVideoStream(Map<String, dynamic> message) {
    Log.debug("[$_clientId]: (client) receive on_add_video_stream: $message");
    var id = message["uid"];
    if (id == null) {
      return;
    }
    var stream = remoteStreams[id];
    if (stream == null) {
      stream = AgoraStream({
        "video": message["video"],
        "audio": message["audio"],
        "streamId": message["uid"],
        "isLocal": false,
      });
      stream.gw = _gatewayClient;
      remoteStreams[stream.getId()] = stream;
      this.emit("stream-added", {"stream": stream});
    } else {
      stream.video = true;
    }
  }

  void _onRemoveStream(Map<String, dynamic> message) {
    Log.debug("[$_clientId]: (client) receive onRemoveStream: $message");
    var id = message["uid"];
    var stream = remoteStreams[id];
    _gatewayClient?.closeSubConnection(id);
    if(stream.isPlaying()) {
      stream.close();
    }
    if (stream != null) {
      remoteStreams.remove(id);
      this.emit("stream-removed", {"stream": stream});
    }
  }

  void _onPeerLeave(Map<String, dynamic> message) {
    Log.debug("[$_clientId]: (client) receive onPeerLeave: $message");
    var id = message["uid"];
    var stream = remoteStreams[id];
    _gatewayClient?.closeSubConnection(id);
    if (stream != null) {
      remoteStreams.remove(id);
    }
    this.emit("peer-leave", {"uid": id});
  }

  void _onP2PLost(Map<String, dynamic> message) async {
    Log.debug("[$_clientId]: (client) receive onP2PLost: $message");
    var p2pId = message["p2pid"];
    if(p2pId == null) {
      Log.debug("[$_clientId]: (client) _onP2PLost: p2pId is null");
      return ;
    }
    var pc = _gatewayClient?.pcController?.getPeerConnectionByP2PId(p2pId);
    if(pc == null) {
      Log.debug("[$_clientId]: (client) _onP2PLost: pc is null, p2pId: $p2pId");
      return ;
    }
    var stream = pc.stream;
    if(stream == null) {
      Log.debug("[$_clientId]: (client) _onP2PLost: stream is null, p2pId: $p2pId");
      return ;
    }
    
    var reconSuccess = true;
    Log.debug("[$_clientId]: (client) stream reconnect start: ${stream.getId()}");
    this.emit("stream-reconnect-start", {"uid": stream.getId()});
    try {
      if(stream.isLocal == true) {
        await unpublish(stream);
        await publish(stream);
      } else {
        await unsubscribe(stream);
        await subscribe(stream);
        if(stream.isPlaying()) {
          stream.render.srcObject = stream.mediaStream;
        }
      }
    } catch (e) {
      reconSuccess = false;
    } finally {
      this.emit("stream-reconnect-end", {"uid": stream.getId(), "success": reconSuccess});
    }
  }

  void _listenGatewayEvent() {
    _gwEvent = _gatewayClient.gwEvent;
    _gwEvent.listen((event) {
      var type = event["_type"];
      Map<String, dynamic> message = event["_message"];
      switch (type) {
        case 'on_add_audio_stream':
          _onAddAudioStream(message);
          break;

        case 'on_add_video_stream':
          _onAddVideoStream(message);
          break;

        case 'on_remove_stream':
          _onRemoveStream(message);
          break;

        case 'on_user_offline':
          _onPeerLeave(message);
          break;

        case 'on_p2p_lost':
          _onP2PLost(message);
          break;
      }
    });
  }

  void _reconnect() async{
    if(_gatewayClient == null) {
      Log.debug("[$_clientId]: (client) gatewayClient should not be null when rejoin");
      throw "(client) gatewayClient should not be null";
    }
    if(_gatewayClient.state != DISCONNECTED) {
      Log.debug("[$_clientId]: (client) _gatewayClient.state should be DISCONNECTED when rejoin");
      throw "(client) gatewayClient.state should be DISCONNECTED";
    }
    remoteStreams.forEach((int uid, AgoraStream stream) {
      if(stream.isPlaying()) {
        stream.close();
      }
      this.emit("stream-removed", {"stream": stream});
    });
    remoteStreams = {};
    await _rejoin();
    await _republish();
  }

  void _onConnectionStateChange(String prvState, String curState) async {
    Log.debug("[$_clientId]: (client) connection state change: $prvState -> $curState");
    this.emit("connection-state-change", {"prevState": prvState, "curState": curState});
    if(curState == DISCONNECTED && reconnect == true) {
      _reconnect();
    }
  }
}

==> agora_flutter_webrtc-0.0.3.tar.gz: lib/agora_client.dart <==
import 'dart:async';

import './src/utils/index.dart';
import './src/utils/logger.dart';
import './src/utils/report.dart';

import './src/core/gateway_client.dart';
import './agora_stream.dart';
import './src/utils/events.dart';

class AgoraClient extends EventEmitter {
  /// The channel profile.
  /// 
  /// Agora Web SDK needs to know the application scenario to apply different optimization methods.
  /// 
  /// Currently SDK supports the following channel profiles:
  /// - "live": Sets the channel profile as live broadcast.
  /// - "rtc": Sets the channel profile as communication. This is used in one-on-one calls or group calls, where all users in the channel can talk freely.
  String mode;

  /// The codec the Web browser uses for encoding.
  /// 
  /// - "vp8": Sets the browser to use VP8 for encoding.
  /// - "h264": Sets the browser to use H.264 for encoding.
  String codec;

  /// App IDs are issued to app developers by Agora to identify the projects and organizations. 
  /// 
  /// After signing up at [Agora Dashboard](https://dashboard.agora.io), you can create multiple projects, and each project will have a unique App ID. See [Getting an App ID](https://docs.agora.io/en/Agora%20Platform/token#getting-an-app-id).
  String appId;

  /// A string that provides a unique channel name for the Agora session. 
  /// 
  /// The length must be within 64 bytes. Supported character scopes:
  /// - 26 lowercase English letters a-z
  /// - 26 uppercase English letters A-Z
  /// - 10 numbers 0-9
  /// - Space
  /// - "!", "#", "$", "%", "&", "(", ")", "+", "-", ":", ";", "<", "=", ".", ">", "?", "@", "[", "]", "^", "_", "{", "}", "|", "~", ","
  String channel;

  /// Set if SDK auto reconnect when disconnected
  bool reconnect = true;
  
  String _token;
  String _sid;
  String _clientId;
  int _uid;
  int _cid;
  int _reconnectCount = 0;

  GatewayClient _gatewayClient;
  Stream _gwEvent;
  Timer _reconTimer;

  /// Stream in local , a [AgoraClient] only have one localstream after publish
  AgoraStream localStream;

  /// All remote streams in the channel
  Map<int, AgoraStream> remoteStreams = {};

  /// The user ID, an integer or a string, ASCII characters only. Ensure this ID is unique
  get uid => _uid;

  /// Creates a Client Object
  /// 
  /// This method creates and returns a client object. You can only call this method once each call session.
  AgoraClient({this.appId, this.mode, this.codec}) {
    _clientId = Utils.getUUID().replaceAll(new RegExp(r'-'), '');
    _sid = Utils.getUUID().replaceAll(new RegExp(r'-'), ''); // generate sid
    EventReportManager.register(sid: _sid);
  }

  /// This method joins an AgoraRTC channel.
  /// 
  /// When joining the channel, the [AgoraClient].on("connection-state-change") callbacks are triggered on the local client.
  Future<int> join(String token, String channel, int uid) async {
    Log.debug("[$_clientId]: (client) join with: $token, $channel, $uid");
    _gatewayClient = GatewayClient(
        appId: appId, mode: mode, codec: codec, clientId: _clientId, sid: _sid);
    _gatewayClient.onConnectionStateChange = _onConnectionStateChange;
    this._token = token;
    this.channel = channel;
    var joinRes = await _gatewayClient.join(null, channel, uid);
    _listenGatewayEvent();

    this._uid = joinRes["uid"];
    this._cid = joinRes["cid"];
    
    EventReportManager.report(_sid, "join", {
      "cid": _cid,
      "uid": uid,
      "cname": channel,
      "processId": Log.processId
    });

    return uid;
  }

  /// Publishes a Local Stream
  /// 
  /// This method publishes a local [stream] to the SD-RTN.
  /// When the [stream] is published, the [AgoraClient].on("stream-added") callback is triggered on the remote client.
  Future<AgoraStream> publish(AgoraStream stream) async {
    Log.debug("[$_clientId]: (client) publish");
    stream.streamId = uid;
    localStream = stream;

    EventReportManager.report(_sid, "publish", {"streamId": uid});

    return _gatewayClient.publish(stream);
  }

  /// Unpublishes the Local Stream
  /// 
  /// This method unpublishes the local [stream].
  /// When the [stream] is unpublished, the [AgoraClient].on("stream-removed") callback is triggered on the remote client.
  Future<AgoraStream> unpublish(AgoraStream stream) async {
    Log.debug("[$_clientId]: (client) unpublish");
    _gatewayClient.unpublish(stream);
    stream.streamId = null;
    localStream = null;
    return stream;
  }

  /// Subscribes to a Remote Stream
  /// 
  /// This method enables a user to subscribe to a remote [stream].
  Future<AgoraStream> subscribe(AgoraStream stream) async {
    Log.debug("[$_clientId]: (client) subscribe");
    return _gatewayClient.subscribe(stream);
  }

  /// Unsubscribes from the Remote Stream
  /// 
  /// This method enables the user to unsubscribe from the remote [stream].
  Future<AgoraStream> unsubscribe(AgoraStream stream) async {
    Log.debug("[$_clientId]: (client) unsubscribe");
    return _gatewayClient.unsubscribe(stream);
  }

  /// Leaves an AgoraRTC Channel
  /// 
  /// This method enables a user to leave a channel.
  /// When leaving the channel, the [AgoraClient].on("connection-state-change") callback is triggered on the local client.
  void leave() {
    Log.debug("[$_clientId]: (client) leave");
    _reconTimer?.cancel();
    _gatewayClient?.leave();
  }

  Future _rejoin() async {
    Log.debug("[$_clientId]: (client) rejoin");
    Completer c = Completer<dynamic>();
    var deleyReconTime = _reconnectCount * 2 < 10 ? _reconnectCount * 2 : 30;
    _reconnectCount++;
    _reconTimer = new Timer(Duration(seconds: deleyReconTime), () {
      join(_token, channel, uid).then((res) {
        Log.debug("[$_clientId]: (client) rejoin success");
        c.complete(res);
      }).catchError((err) {
        Log.debug("[$_clientId]: (client) rejoin error");
        _rejoin().then((res) {
          c.complete(res);
        });
      });
    });
    return c.future;
  }

  Future _republish() async {
    Log.debug("[$_clientId]: (client) rejoin");
    return publish(localStream);
  }

  void _onAddAudioStream(Map<String, dynamic> message) {
    Log.debug("[$_clientId]: (client) receive on_add_audio_stream: $message");
    var stream = AgoraStream({
      "video": message["video"],
      "audio": message["audio"],
      "streamId": message["uid"],
      "isLocal": false,
    });
    stream.gw = _gatewayClient;
    remoteStreams[stream.getId()] = stream;
    this.emit("stream-added", {"stream": stream});
  }

  void _onAddVideoStream(Map<String, dynamic> message) {
    Log.debug("[$_clientId]: (client) receive on_add_video_stream: $message");
    var id = message["uid"];
    if (id == null) {
      return;
    }
    var stream = remoteStreams[id];
    if (stream == null) {
      stream = AgoraStream({
        "video": message["video"],
        "audio": message["audio"],
        "streamId": message["uid"],
        "isLocal": false,
      });
      stream.gw = _gatewayClient;
      remoteStreams[stream.getId()] = stream;
      this.emit("stream-added", {"stream": stream});
    } else {
      stream.video = true;
    }
  }

  void _onRemoveStream(Map<String, dynamic> message) {
    Log.debug("[$_clientId]: (client) receive onRemoveStream: $message");
    var id = message["uid"];
    var stream = remoteStreams[id];
    _gatewayClient?.closeSubConnection(id);
    if(stream.isPlaying()) {
      stream.close();
    }
    if (stream != null) {
      remoteStreams.remove(id);
      this.emit("stream-removed", {"stream": stream});
    }
  }

  void _onPeerLeave(Map<String, dynamic> message) {
    Log.debug("[$_clientId]: (client) receive onPeerLeave: $message");
    var id = message["uid"];
    var stream = remoteStreams[id];
    _gatewayClient?.closeSubConnection(id);
    if (stream != null) {
      remoteStreams.remove(id);
    }
    this.emit("peer-leave", {"uid": id});
  }

  void _onP2PLost(Map<String, dynamic> message) async {
    Log.debug("[$_clientId]: (client) receive onP2PLost: $message");
    var p2pId = message["p2pid"];
    if(p2pId == null) {
      Log.debug("[$_clientId]: (client) _onP2PLost: p2pId is null");
      return ;
    }
    var pc = _gatewayClient?.pcController?.getPeerConnectionByP2PId(p2pId);
    if(pc == null) {
      Log.debug("[$_clientId]: (client) _onP2PLost: pc is null, p2pId: $p2pId");
      return ;
    }
    var stream = pc.stream;
    if(stream == null) {
      Log.debug("[$_clientId]: (client) _onP2PLost: stream is null, p2pId: $p2pId");
      return ;
    }
    
    var reconSuccess = true;
    Log.debug("[$_clientId]: (client) stream reconnect start: ${stream.getId()}");
    this.emit("stream-reconnect-start", {"uid": stream.getId()});
    try {
      if(stream.isLocal == true) {
        await unpublish(stream);
        await publish(stream);
      } else {
        await unsubscribe(stream);
        await subscribe(stream);
        if(stream.isPlaying()) {
          stream.render.srcObject = stream.mediaStream;
        }
      }
    } catch (e) {
      reconSuccess = false;
    } finally {
      this.emit("stream-reconnect-end", {"uid": stream.getId(), "success": reconSuccess});
    }
  }

  void _listenGatewayEvent() {
    _gwEvent = _gatewayClient.gwEvent;
    _gwEvent.listen((event) {
      var type = event["_type"];
      Map<String, dynamic> message = event["_message"];
      switch (type) {
        case 'on_add_audio_stream':
          _onAddAudioStream(message);
          break;

        case 'on_add_video_stream':
          _onAddVideoStream(message);
          break;

        case 'on_remove_stream':
          _onRemoveStream(message);
          break;

        case 'on_user_offline':
          _onPeerLeave(message);
          break;

        case 'on_p2p_lost':
          _onP2PLost(message);
          break;
      }
    });
  }

  void _reconnect() async{
    if(_gatewayClient == null) {
      Log.debug("[$_clientId]: (client) gatewayClient should not be null when rejoin");
      throw "(client) gatewayClient should not be null";
    }
    if(_gatewayClient.state != DISCONNECTED) {
      Log.debug("[$_clientId]: (client) _gatewayClient.state should be DISCONNECTED when rejoin");
      throw "(client) gatewayClient.state should be DISCONNECTED";
    }
    remoteStreams.forEach((int uid, AgoraStream stream) {
      if(stream.isPlaying()) {
        stream.close();
      }
      this.emit("stream-removed", {"stream": stream});
    });
    remoteStreams = {};
    await _rejoin();
    await _republish();
  }

  void _onConnectionStateChange(String prvState, String curState) async {
    Log.debug("[$_clientId]: (client) connection state change: $prvState -> $curState");
    this.emit("connection-state-change", {"prevState": prvState, "curState": curState});
    if(curState == DISCONNECTED && reconnect == true) {
      _reconnect();
    }
  }
}
